/* File "datagram.cpp", Light Sockets - Copyright (C) Matt Slot, 1996         */
/* Datagram dispatch for the Light Sockets networking abstraction routines.   */

#include "stddebug.h"
#include "stdtypes.h"

#include "datagram.h"
#include "netstack.h"
#include "socket.h"

/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */

DatagramSocket::DatagramSocket() {
	SocketResult &error = instError;

	error = eSocketNoError;

	/* Structure initialization */
	writeProc = WriteStub;
	writeRefcon = 0;
	readProc = ReadStub;
	readRefcon = 0;
	}

/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */

DatagramSocket::~DatagramSocket() {
	DatagramSocket::Dispose();
	}

/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
#pragma mark -

SocketResult DatagramSocket::Create(SocketType socketType) {
	SocketResult &error = instError;
	
	/* Parameter validation */
	qThrowIfError(error, 0);

	/* Chained function call */
	qThrowIfError(Socket::Create(socketType), 0);

/* CATCH */
	qCatch();
	
	if (error) 
		Dispose();

	return(error);
	}

/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */

SocketResult DatagramSocket::Tickle() {
	SocketResult &error = instError;
	
	/* Parameter validation */
	qThrowIfError(error, 0);

	/* Chained function call */
	qThrowIfError(Socket::Tickle(), 0);
	

/* CATCH */
	qCatch();

	return(error);
	}
	
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */

SocketResult DatagramSocket::Dispose() {
	SocketResult &error = instError;
	
	/* Chained function call */
	qThrowIfError(Socket::Dispose(), 0);

/* CATCH */
	qCatch();

	return(error);
	}
	
	
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
#pragma mark -

SInt32 DatagramSocket::Write(Byte8 *dataPtr, UInt32 dataLen,
		SocketAddressPtr address) {
	SocketResult &error = instError;
	
	/* Parameter validation */
	qThrowIfError(error, 0);
	qAssertIfNull(stack, eSocketErrInternal, kSocketErrInternalStr);
		
	qThrowIfError(stack->DoDatagramWrite(dataPtr, dataLen, address), 0);

/* CATCH */
	qCatch();
	return(error);
	}

/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
#pragma mark -

SInt32 DatagramSocket::GetWriteProc(DatagramWriteProc *proc, void **refcon) {
	SocketResult &error = instError;
	
	/* Parameter validation */
	qThrowIfError(error, 0);
	qThrowIfNull(proc, eSocketErrBadParam, kSocketErrBadParamStr);
	qThrowIfNull(refcon, eSocketErrBadParam, kSocketErrBadParamStr);
		
	*proc = writeProc;
	*refcon = writeRefcon;


/* CATCH */
	qCatch();
	return(error);
	}

/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */

SInt32 DatagramSocket::SetWriteProc(DatagramWriteProc proc, void *refcon) {
	SocketResult &error = instError;
	
	/* Parameter validation */
	qThrowIfError(error, 0);

	writeProc = (proc) ? proc : WriteStub;
	writeRefcon = (proc) ? refcon : 0;


/* CATCH */
	qCatch();
	return(error);
	}

/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */

SocketResult DatagramSocket::CallWriteProc(Byte8 *dataPtr, UInt32 dataLen,
		SocketAddressPtr address) {
	SocketResult &error = instError;
	
	/* Parameter validation */
	qThrowIfError(error, 0);
	qAssertIfNull(writeProc, eSocketErrInternal, kSocketErrInternalStr);

	/* Make the function call */
	qThrowIfError((*writeProc)(this, writeRefcon, dataPtr, dataLen, address),0);


/* CATCH */
	qCatch();
	return(error);
	}

/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */

SocketResult DatagramSocket::WriteStub(DatagramSocketRef socket, void *refcon,
		Byte8 *dataPtr, UInt32 dataLen, SocketAddressPtr address) {
	SocketResult &error = socket->instError;
	
	/* Parameter validation */
	qThrowIfError(error, 0);
	qAssertIfNull(socket, eSocketErrInternal, kSocketErrInternalStr);


/* CATCH */
	qCatch();
	return(error);
	}

/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
#pragma mark -

SInt32 DatagramSocket::GetReadProc(DatagramReadProc *proc, void **refcon) {
	SocketResult &error = instError;
	
	/* Parameter validation */
	qThrowIfError(error, 0);
	qThrowIfNull(proc, eSocketErrBadParam, kSocketErrBadParamStr);
	qThrowIfNull(refcon, eSocketErrBadParam, kSocketErrBadParamStr);
		
	*proc = readProc;
	*refcon = readRefcon;


/* CATCH */
	qCatch();
	return(error);
	}

/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */

SInt32 DatagramSocket::SetReadProc(DatagramReadProc proc, void *refcon) {
	SocketResult &error = instError;
	
	/* Parameter validation */
	qThrowIfError(error, 0);

	readProc = (proc) ? proc : ReadStub;
	readRefcon = (proc) ? refcon : 0;


/* CATCH */
	qCatch();
	return(error);
	}

/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */

SocketResult DatagramSocket::CallReadProc(Byte8 *dataPtr, UInt32 dataLen,
		SocketAddressPtr address) {
	SocketResult &error = instError;
	
	/* Parameter validation */
	qThrowIfError(error, 0);
	qAssertIfNull(readProc, eSocketErrInternal, kSocketErrInternalStr);

	/* Make the function call */
	qThrowIfError((*readProc)(this, readRefcon, dataPtr, dataLen, address), 0);


/* CATCH */
	qCatch();
	return(error);
	}

/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */

SocketResult DatagramSocket::ReadStub(DatagramSocketRef socket, void *refcon,
		Byte8 *dataPtr, UInt32 dataLen, SocketAddressPtr address) {
	SocketResult &error = socket->instError;
	
	/* Parameter validation */
	qThrowIfError(error, 0);
	qAssertIfNull(socket, eSocketErrInternal, kSocketErrInternalStr);


/* CATCH */
	qCatch();
	return(error);
	}

