/* File "reggie.h", ISR/Reggie - Copyright (C) Matt Slot + Mike Monan, 1996   */
/* Fundamental networking header for the Internet Service Registry (reggie)   */

#ifndef __REGGIE_HEADER__
#define __REGGIE_HEADER__

/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* Functional Summary */

/*
The networking for reggie has been designed with the following goals in mind:
	1) stateless TRACKER operation,
	2) low overhead server registration,
	3) low overhead client access, and
	4) simplicity.

The reggie model consists of 4 types of functional units:
	1) TRACKER - The reggie service that maintains a database of existing
	   services, moderates the SERVER registration and unregistration
	   process, and answers CLIENT queries for current service lists.
	2) SERVER - Some software providing a networked service that it wishes
	   to announce using the reggie protocol. The SERVER is responsible for
	   registering and periodically reminding the TRACKER that is alive. It
	   is also responsible for unregistering, however a TRACKER provides a
	   simple timeout facility.
	3) CLIENT - A user driven package which queries a TRACKER to locate a
	   SERVER and connect the user to its service.
	4) PROXY - A third party service which informs a given TRACKER of a new 
	   SERVER, often based on other information provided by special software,
	   configuration files, or (in the future) other TRACKERS.
	   

There are 3 type of transaction sequences currently defined:

	1)  <Any Host>                          TRACKER,SERVER,PROXY
                        ---- PING ---->
                       <---- ACK -----

	-----

	2a) SERVER,PROXY                        TRACKER
                        -- REGISTER -->
                       <---- ACK -----

	2b) SERVER,PROXY                        TRACKER
                        - UNREGISTER ->
                       <---- ACK -----

	-----

	3a) CLIENT                              TRACKER
                        --- QUERY ---->
                       <---- ACK -----

	3a) CLIENT                              TRACKER
                        --- QUERY ---->
                       <--- ANSWER ---
                       <--- ANSWER ---
                             ...
                       <--- ANSWER ---

Other notes:

	Packets that contain string information, the strings are stored as packed,
	byte-aligned, 0-terminated strings. Each string and its terminator must
	fit within the (maximum) buffer size described.

	A SERVER or PROXY will continue to "tickle" the TRACKER with duplicate (or
	modified) register packets at approximately half the announced TTL. The
	TRACKER is responsible for storing any changes in the register information
	within its database.
	
	A TRACKER will not expire a service entry from its database in response to
	an unregister request from a host/port that is not the SERVER or the PROXY
	(if there is one).
	
	A query mask is composed of ASCII characters, with the following wildcard
	interpretations: '*'=zero or more characters, '?'-exactly one character, 
	and '\'=escape character ("\*", "\?", "\\"). Under consideration is a ',' 
	operator that performs a Boolean AND between two masks. (Boolean OR can be
	simulated by repeated queries).
	
*/

/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* Include Files */

#include "stdtypes.h"

/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* Preprocessor Defines */

#define kTrackerPort	13131


/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* Structure/Class Declarations */


typedef struct ServiceAddress {
	UInt8			hostProtFamily;	/* 6=PF_TCP, 17=PF_UDP */
	UInt8			hostAddrFamily;	/* 2=AF_INET */
	UInt16			hostPort;
	UInt32			hostAddress;
	UInt8			data[8];
	} ServiceAddress, *ServiceAddressPtr;


typedef union Packet {
	struct {
		UInt32			transactionType;
		UInt32			transactionID;
		UInt32          reserved[2];
		} any;
	struct {
		UInt32			transactionType;
		UInt32			transactionID;
		UInt32			reserved[2];
		} ack;
	struct {
		UInt32			transactionType;
		UInt32			transactionID;
		UInt32			reserved[2];
		} ping;
	struct {
		UInt32			transactionType;
		UInt32			transactionID;
		UInt32			reserved[1];
		UInt32			serviceTTL;
		ServiceAddress	serviceAddr;
		Char8			serviceType[20];
		Char8			serviceName[256];
		Char8			serviceInfo[256];
		} reg;
	struct {
		UInt32			transactionType;
		UInt32			transactionID;
		UInt32			reserved[4];
		UInt32			indexStart;
		UInt32			indexEnd;
		Char8			serviceType[20];
		Char8			serviceNameMask[256];
		Char8			serviceInfoMask[256];
		} query;
	struct {
		UInt32			transactionType;
		UInt32			transactionID;
		UInt32			reserved[1];
		UInt32			serviceIndex;
		ServiceAddress	serviceAddr;
		Char8			serviceType[20];
		Char8			serviceName[256];
		Char8			serviceInfo[256];
		} answer;
	} Packet, *PacketPtr;

enum {
	eTransactionAck=0,
	eTransactionPing,
	eTransactionRegister,
	eTransactionUnregister,
	eTransactionQuery,
	eTransactionAnswer,

	eServiceTransitory=60*2,
	eServiceShortLived=60*30,
	eServiceLongLived=60*60*8,
	eServicePermanent=0
	};

/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* **** **** **** **** **** **** **** **** **** **** **** **** **** **** **** */
/* Function Prototypes */

#endif /* __REGGIE_HEADER__ */


